@@ -49,6 +49,8 @@
 
 /* IO Stream types */
 #define pxfIOFile 1
+/* pxfIOGsf is defined as 2 in paradox-gsf.h */
+#define pxfIOStream 3
 
 /* Field types */
 #define pxfAlpha        0x01
@@ -120,19 +122,6 @@ struct px_head {
 	struct px_field *px_fields;
 };
 
-struct px_stream {
-	int type;        /* set to pxfIOFile | pxfIOGsf */
-	int mode;        /* set to pxfFileRead | pxfFileWrite */
-	int close;       /* set to true if stream must be closed */
-	union {
-		FILE *fp;
-#if PX_HAVE_GSF
-		GsfInput *gsfin;
-		GsfOutput *gsfout;
-#endif
-	} s;
-};
-
 typedef struct px_doc pxdoc_t;
 typedef struct px_datablockinfo pxdatablockinfo_t;
 typedef struct px_blob pxblob_t;
@@ -142,6 +131,24 @@ typedef struct px_pindex pxpindex_t;
 typedef struct px_stream pxstream_t;
 typedef struct mb_head mbhead_t;
 
+struct px_stream {
+	int type;        /* set to pxfIOFile | pxfIOGsf | pxfIOStream*/
+	int mode;        /* set to pxfFileRead | pxfFileWrite */
+	int close;       /* set to true if stream must be closed */
+	union {
+		FILE *fp;
+		void *stream;
+#if PX_HAVE_GSF
+		GsfInput *gsfin;
+		GsfOutput *gsfout;
+#endif
+	} s;
+	char * (*read)(pxdoc_t *p, void *stream, size_t numbytes, void *buffer);
+	size_t (*seek)(pxdoc_t *p, void *stream, size_t offset, int whence);
+	size_t (*tell)(pxdoc_t *p, void *stream);
+	size_t (*write)(pxdoc_t *p, void *stream, const char *buffer, size_t numbytes);
+};
+
 struct px_doc {
 	/* database file */
 //	FILE *px_fp;       /* File pointer of file */
@@ -166,7 +173,7 @@ struct px_doc {
 	/* record accounting */
 	int last_position; /* Position (starting at 0) where last record was put. */
 
-	int in_error;
+	int warnings;      /* Turn of/off output of warnings */
 
 	/* output function */
 	size_t (*writeproc)(pxdoc_t *p, void *data, size_t size);
@@ -366,6 +373,9 @@ PX_set_tablename(pxdoc_t *pxdoc, const char *tablename);
 PXLIB_API int PXLIB_CALL
 PX_set_blob_file(pxdoc_t *pxdoc, const char *filename);
 
+PXLIB_API int PXLIB_CALL
+PX_set_blob_fp(pxdoc_t *pxdoc, FILE *fp);
+
 PXLIB_API int PXLIB_CALL
 PX_has_blob_file(pxdoc_t *pxdoc);
 
@@ -378,6 +388,12 @@ PX_open_blob_fp(pxblob_t *pxdoc, FILE *fp);
 PXLIB_API int PXLIB_CALL
 PX_open_blob_file(pxblob_t *pxdoc, const char *filename);
 
+PXLIB_API int PXLIB_CALL
+PX_create_blob_fp(pxblob_t *pxdoc, FILE *fp);
+
+PXLIB_API int PXLIB_CALL
+PX_create_blob_file(pxblob_t *pxblob, const char *filename);
+
 PXLIB_API void PXLIB_CALL
 PX_close_blob(pxblob_t *pxdoc);
 
@@ -444,6 +460,9 @@ PX_put_data_byte(pxdoc_t *pxdoc, char *data, int len, char value);
 PXLIB_API void PXLIB_CALL
 PX_put_data_bcd(pxdoc_t *pxdoc, char *data, int len, char *value);
 
+PXLIB_API int PXLIB_CALL
+PX_put_data_blob(pxdoc_t *pxdoc, char *data, int len, char *value, int valuelen);
+
 PXLIB_API void PXLIB_CALL
 PX_SdnToGregorian(long int sdn, int *pYear, int *pMonth, int *pDay);
 
