@@ -1,5 +1,5 @@
 /*
- * pslibversion.h : compile-time version informations for the pslib.
+ * pxversion.h : compile-time version informations for the pslib.
  *
  * See Copyright for the status of this software.
  *
@@ -17,18 +17,18 @@ extern "C" {
  * use those to be sure nothing nasty will happen if
  * your library and includes mismatch
  */
-#define PXLIB_DOTTED_VERSION "0.4.3"
+#define PXLIB_DOTTED_VERSION "0.4.4"
 #define PXLIB_MAJOR_VERSION 0
 #define PXLIB_MINOR_VERSION 4
-#define PXLIB_MICRO_VERSION 3
+#define PXLIB_MICRO_VERSION 4
 
 /*
  * Whether Debugging module is configured in
  */
 #if 1
-#define PSLIB_DEBUG_ENABLED
+#define PXLIB_DEBUG_ENABLED
 #else
-#define PSLIB_DEBUG_DISABLED
+#define PXLIB_DEBUG_DISABLED
 #endif
 
 #ifdef __cplusplus
