@@ -21,6 +21,8 @@
 #include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
+#include <errno.h>
 #if defined(WIN32) || defined(OS2)
 #include <fcntl.h>
 #endif
@@ -39,6 +41,7 @@
 #include "px_io.h"
 #include "px_error.h"
 #include "px_misc.h"
+#include "px_encode.h"
 
 #define max(a,b) ((a)>(b) ? (a) : (b))
 
@@ -214,6 +217,39 @@ PX_get_opaque(pxdoc_t *pxdoc) {
 }
 /* }}} */
 
+/* PX_set_io_stream() {{{
+ * Sets file access routines.
+ */
+PXLIB_API int PXLIB_CALL
+PX_set_io_stream(pxdoc_t *pxdoc,
+                 char * (*readproc)(pxdoc_t *p, void *stream, size_t numbytes, void *buffer),
+                 size_t (*writeproc)(pxdoc_t *p, void *stream, const char *data, size_t len),
+                 size_t (*seekproc)(pxdoc_t *p, void *stream, size_t offset, int whence),
+                 size_t (*tellproc)(pxdoc_t *p, void *stream)
+				 ) {
+	pxstream_t *pxs;
+
+	if(pxdoc == NULL) {
+		px_error(pxdoc, PX_RuntimeError, _("Did not pass a paradox database."));
+		return -1;
+	}
+
+	if(NULL == (pxs = pxdoc->malloc(pxdoc, sizeof(pxstream_t), _("Allocate memory for io stream.")))) {
+		px_error(pxdoc, PX_MemoryError, _("Could not allocate memory for io stream."));
+		return -1;
+	}
+	
+	pxs->read = readproc;
+	pxs->seek = seekproc;
+	pxs->tell = tellproc;
+	pxs->write = writeproc;
+
+	pxdoc->px_stream = pxs;
+
+	return(0);
+}
+/* }}} */
+
 /* build_primary_index() {{{
  * Build a primary index.
  */
@@ -261,13 +297,57 @@ static int build_primary_index(pxdoc_t *pxdoc) {
 		blockcount++;
 		blocknumber = get_short_le((char *) &datablockhead.nextBlock);
 	}
+	return 0;
 }
 /* }}} */
 
 
+/* PX_open_stream() {{{
+ * Read from a Paradox DB file, which has an already open.
+ */
+PXLIB_API int PXLIB_CALL
+PX_open_stream(pxdoc_t *pxdoc, void *stream) {
+	pxhead_t *pxh;
+	pxstream_t *pxs;
+
+	if(pxdoc == NULL) {
+		px_error(pxdoc, PX_RuntimeError, _("Did not pass a paradox database."));
+		return -1;
+	}
+
+	if(pxdoc->px_stream == NULL) {
+		px_error(pxdoc, PX_RuntimeError, _("Paradox database has no stream."));
+		return -1;
+	}
+	pxs = pxdoc->px_stream;
+
+	pxdoc->px_stream->type = pxfIOStream;
+	pxdoc->px_stream->mode = pxfFileRead;
+	pxdoc->px_stream->close = px_false;
+	pxdoc->px_stream->s.stream = stream;
+
+	if((pxdoc->px_head = get_px_head(pxdoc, pxs)) == NULL) {
+		px_error(pxdoc, PX_RuntimeError, _("Unable to get header."));
+		return -1;
+	}
+
+	/* Build primary index. This index misses all index blocks with a level
+	 * greater than 1. Since they are not used currently this is of no harm.
+	 */
+	pxh = pxdoc->px_head;
+	if(pxh->px_filetype == pxfFileTypIndexDB ||
+	   pxh->px_filetype == pxfFileTypNonIndexDB) {
+		if(build_primary_index(pxdoc) < 0) {
+			return -1;
+		}
+	}
+	return 0;
+}
+/* }}} */
+
 #if PX_HAVE_GSF
 /* PX_open_gsf() {{{
- * Read from a Paradox DB file, which has already been opend with gsf.
+ * Read from a Paradox DB file, which has already been opened with gsf.
  */
 PXLIB_API int PXLIB_CALL
 PX_open_gsf(pxdoc_t *pxdoc, GsfInput *gsf) {
@@ -548,23 +628,32 @@ PX_set_value(pxdoc_t *pxdoc, const char *name, float value) {
 		return -1;
 	}
 
+	if(!(pxdoc->px_stream->mode & pxfFileWrite)) {
+		px_error(pxdoc, PX_Warning, _("File is not writable. Setting '%s' has no effect."), name);
+		return -1;
+	}
+
 	if(strcmp(name, "numprimkeys") == 0) {
-		if(pxdoc->px_stream->mode & pxfFileWrite) {
-			if(value < 0) {
-				px_error(pxdoc, PX_Warning, _("Number of primary keys must be greater or equal 0."), name);
-				return -1;
-			}
-			pxdoc->px_head->px_primarykeyfields = (int) value;
-			if(value == 0) {
-				pxdoc->px_head->px_filetype = pxfFileTypNonIndexDB;
-			} else {
-				pxdoc->px_head->px_filetype = pxfFileTypIndexDB;
-			}
-			if(put_px_head(pxdoc, pxdoc->px_head, pxdoc->px_stream) < 0) {
-				return -1;
-			}
+		if(value < 0) {
+			px_error(pxdoc, PX_Warning, _("Number of primary keys must be greater or equal 0."), name);
+			return -1;
+		}
+		pxdoc->px_head->px_primarykeyfields = (int) value;
+		if(value == 0) {
+			pxdoc->px_head->px_filetype = pxfFileTypNonIndexDB;
 		} else {
-			px_error(pxdoc, PX_Warning, _("File is not writable. Setting '%s' has no effect."), name);
+			pxdoc->px_head->px_filetype = pxfFileTypIndexDB;
+		}
+		if(put_px_head(pxdoc, pxdoc->px_head, pxdoc->px_stream) < 0) {
+			return -1;
+		}
+	} else if(strcmp(name, "codepage") == 0) {
+		if(value <= 0) {
+			px_error(pxdoc, PX_Warning, _("codepage must be greater 0."), name);
+			return -1;
+		}
+		pxdoc->px_head->px_doscodepage = (int) value;
+		if(put_px_head(pxdoc, pxdoc->px_head, pxdoc->px_stream) < 0) {
 			return -1;
 		}
 	} else {
@@ -588,6 +677,51 @@ PX_get_value(pxdoc_t *pxdoc, const char *name, float *value) {
 	if(strcmp(name, "numprimkeys") == 0) {
 		*value = (float) pxdoc->px_head->px_primarykeyfields;
 		return(0);
+	} else if(strcmp(name, "filetype") == 0) {
+		*value = (float) pxdoc->px_head->px_filetype;
+		return(0);
+	} else if(strcmp(name, "numfields") == 0) {
+		*value = (float) pxdoc->px_head->px_numfields;
+		return(0);
+	} else if(strcmp(name, "primarykeyfields") == 0) {
+		*value = (float) pxdoc->px_head->px_primarykeyfields;
+		return(0);
+	} else if(strcmp(name, "numrecords") == 0) {
+		*value = (float) pxdoc->px_head->px_numrecords;
+		return(0);
+	} else if(strcmp(name, "recordsize") == 0) {
+		*value = (float) pxdoc->px_head->px_recordsize;
+		return(0);
+	} else if(strcmp(name, "theonumrecords") == 0) {
+		*value = (float) pxdoc->px_head->px_theonumrecords;
+		return(0);
+	} else if(strcmp(name, "fileversion") == 0) {
+		*value = (float) pxdoc->px_head->px_fileversion/10.0;
+		return(0);
+	} else if(strcmp(name, "headersize") == 0) {
+		*value = (float) pxdoc->px_head->px_headersize;
+		return(0);
+	} else if(strcmp(name, "maxtablesize") == 0) {
+		*value = (float) pxdoc->px_head->px_maxtablesize;
+		return(0);
+	} else if(strcmp(name, "numblocks") == 0) {
+		*value = (float) pxdoc->px_head->px_fileblocks;
+		return(0);
+	} else if(strcmp(name, "firstblock") == 0) {
+		*value = (float) pxdoc->px_head->px_firstblock;
+		return(0);
+	} else if(strcmp(name, "lastblock") == 0) {
+		*value = (float) pxdoc->px_head->px_lastblock;
+		return(0);
+	} else if(strcmp(name, "codepage") == 0) {
+		*value = (float) pxdoc->px_head->px_doscodepage;
+		return(0);
+	} else if(strcmp(name, "autoinc") == 0) {
+		*value = (float) pxdoc->px_head->px_autoinc;
+		return(0);
+	} else if(strcmp(name, "sortorder") == 0) {
+		*value = (float) pxdoc->px_head->px_sortorder;
+		return(0);
 	}
 	px_error(pxdoc, PX_Warning, _("No such value name."));
 	return(-2);
@@ -623,6 +757,7 @@ PX_set_parameter(pxdoc_t *pxdoc, const char *name, const char *value) {
 			return -1;
 		}
 	} else if(strcmp(name, "targetencoding") == 0) {
+		int codepage;
 #if PX_USE_RECODE || PX_USE_ICONV
 		if(pxdoc->targetencoding)
 			pxdoc->free(pxdoc, pxdoc->targetencoding);
@@ -636,6 +771,9 @@ PX_set_parameter(pxdoc_t *pxdoc, const char *name, const char *value) {
 #else
 		px_error(pxdoc, PX_RuntimeError, _("Library has not been compiled with support for reencoding."));
 #endif
+		if(sscanf(value, "CP%d", &codepage)) {
+			PX_set_value(pxdoc, "codepage", codepage);
+		}
 	} else if(strcmp(name, "inputencoding") == 0) {
 #if PX_USE_RECODE || PX_USE_ICONV
 		if(pxdoc->inputencoding)
@@ -650,6 +788,12 @@ PX_set_parameter(pxdoc_t *pxdoc, const char *name, const char *value) {
 #else
 		px_error(pxdoc, PX_RuntimeError, _("Library has not been compiled with support for reencoding."));
 #endif
+	} else if(strcmp(name, "warning") == 0) {
+		if(strcmp(value, "true") == 0) {
+			pxdoc->warnings = px_true;
+		} else {
+			pxdoc->warnings = px_false;
+	    }
 	}
 	return 0;
 }
@@ -783,11 +927,11 @@ PX_add_primary_index(pxdoc_t *pxdoc, pxdoc_t *pindex) {
  */
 PXLIB_API int PXLIB_CALL
 PX_read_primary_index(pxdoc_t *pindex) {
-	pxpindex_t *pindex_data, pdata;
+	pxpindex_t *pindex_data;
 	pxhead_t *pxh;
 	pxfield_t *pxf;
 	char *data;
-	int i, j, swap, datalen;
+	int i, j, datalen;
 
 	if(pindex == NULL ||
 	   pindex->px_head == NULL ||
@@ -876,7 +1020,7 @@ PX_read_primary_index(pxdoc_t *pindex) {
 	} else {
 		int firstblock = pindex_data[0].myblocknumber;
 		int numrecords = 0;
-		for(j=0; j<pxh->px_numrecords, pindex_data[j].myblocknumber == firstblock; j++) {
+		for(j=0; j<pxh->px_numrecords && pindex_data[j].myblocknumber == firstblock; j++) {
 			numrecords += pindex_data[j].numrecords;
 			pindex_data[j].level = 2;
 		}
@@ -931,7 +1075,9 @@ PX_write_primary_index(pxdoc_t *pxdoc, pxdoc_t *pxindex) {
 	}
 
 	if(NULL == pxdoc->px_indexdata)
-		build_primary_index(pxdoc);
+		if(build_primary_index(pxdoc) < 0) {
+			return -1;
+		}
 	indexdata = pxdoc->px_indexdata;
 	indexdatalen = pxdoc->px_indexdatalen;
 
@@ -1068,7 +1214,7 @@ px_get_record_pos_with_index(pxdoc_t *pxdoc, int recno, int *deleted, pxdatabloc
  */
 int
 px_get_record_pos(pxdoc_t *pxdoc, int recno, int *deleted, pxdatablockinfo_t *pxdbinfo) {
-	int ret, found, blockcount, blocknumber;
+	int found, blockcount, blocknumber;
 	TDataBlock datablock;
 	pxhead_t *pxh;
 
@@ -1174,7 +1320,7 @@ PX_get_record(pxdoc_t *pxdoc, int recno, char *data) {
  */
 PXLIB_API char* PXLIB_CALL
 PX_get_record2(pxdoc_t *pxdoc, int recno, char *data, int *deleted, pxdatablockinfo_t *pxdbinfo) {
-	int ret, found, blockcount;
+	int ret, found;
 	pxhead_t *pxh;
 	pxdatablockinfo_t tmppxdbinfo;
 
@@ -1381,9 +1527,9 @@ PX_delete(pxdoc_t *pxdoc) {
 		recode_delete_request(pxdoc->in_recode_request);
 #else
 #if PX_USE_ICONV
-	if(pxdoc->out_iconvcd > 0)
+	if((int) pxdoc->out_iconvcd > 0)
 		iconv_close(pxdoc->out_iconvcd);
-	if(pxdoc->in_iconvcd > 0)
+	if((int) pxdoc->in_iconvcd > 0)
 		iconv_close(pxdoc->in_iconvcd);
 #endif
 #endif
@@ -1545,6 +1691,7 @@ PX_get_recordsize(pxdoc_t *pxdoc) {
  */
 PXLIB_API int PXLIB_CALL
 PX_set_targetencoding(pxdoc_t *pxdoc, const char *encoding) {
+	int codepage;
 #if PX_USE_RECODE || PX_USE_ICONV
 	if(pxdoc == NULL) {
 		px_error(pxdoc, PX_RuntimeError, _("Did not pass a paradox database."));
@@ -1572,6 +1719,9 @@ PX_set_targetencoding(pxdoc_t *pxdoc, const char *encoding) {
 	px_error(pxdoc, PX_RuntimeError, _("Library has not been compiled with support for reencoding."));
 	return -2;
 #endif
+	if(sscanf(encoding, "CP%d", &codepage)) {
+		PX_set_value(pxdoc, "codepage", codepage);
+	}
 	return 0;
 }
 /* }}} */
@@ -1726,32 +1876,22 @@ PX_open_blob_file(pxblob_t *pxblob, const char *filename) {
 }
 /* }}} */
 
-/* PX_create_blob_file() {{{
- * Create a new file for blob data with the given filename
+/* PX_create_blob_fp() {{{
+ * Creates a blob with a given already open file pointer
  */
 PXLIB_API int PXLIB_CALL
-PX_create_blob_file(pxblob_t *pxblob, const char *filename) {
-	FILE *fp;
-	mbhead_t *mbh;
+PX_create_blob_fp(pxblob_t *pxblob, FILE *fp) {
 	pxdoc_t *pxdoc;
 	pxstream_t *pxs;
-
-	if(!pxblob) {
-		return(-1);
-	}
+	mbhead_t *mbh;
 
 	if(NULL == (pxdoc = pxblob->pxdoc)) {
 		px_error(pxdoc, PX_RuntimeError, _("No paradox document associated with blob file."));
 		return -1;
 	}
 
-	if((fp = fopen(filename, "w")) == NULL) {
-		px_error(pxdoc, PX_RuntimeError, _("Could not open blob file '%s' for writing."), filename);
-		return -1;
-	}
-
-	if(NULL == (pxs = pxdoc->malloc(pxdoc, sizeof(pxstream_t), _("Allocate memory for io stream.")))) {
-		px_error(pxdoc, PX_MemoryError, _("Could not allocate memory for io stream."));
+	if(NULL == (pxs = pxdoc->malloc(pxdoc, sizeof(pxstream_t), _("Allocate memory for io stream of blob file.")))) {
+		px_error(pxdoc, PX_MemoryError, _("Could not allocate memory for io stream of blob file."));
 		return -1;
 	}
 	pxs->type = pxfIOFile;
@@ -1776,10 +1916,41 @@ PX_create_blob_file(pxblob_t *pxblob, const char *filename) {
 
 	pxblob->mb_head = mbh;
 	pxblob->mb_stream = pxs;
+	pxblob->used_datablocks = 0;
+
+	return(0);
+}
+/* }}} */
+
+/* PX_create_blob_file() {{{
+ * Create a new file for blob data with the given filename
+ */
+PXLIB_API int PXLIB_CALL
+PX_create_blob_file(pxblob_t *pxblob, const char *filename) {
+	FILE *fp;
+	pxdoc_t *pxdoc;
+
+	if(!pxblob) {
+		return(-1);
+	}
+
+	if(NULL == (pxdoc = pxblob->pxdoc)) {
+		px_error(pxdoc, PX_RuntimeError, _("No paradox document associated with blob file."));
+		return -1;
+	}
+
+	if((fp = fopen(filename, "w")) == NULL) {
+		px_error(pxdoc, PX_RuntimeError, _("Could not open blob file '%s' for writing."), filename);
+		return -1;
+	}
+
+	if(0 > PX_create_blob_fp(pxblob, fp)) {
+		fclose(fp);
+		return -1;
+	}
 
 	pxblob->mb_name = px_strdup(pxblob->pxdoc, filename);
 	pxblob->mb_stream->close = px_true;
-	pxblob->used_datablocks = 0;
 
 	return 0;
 }
@@ -1829,7 +2000,13 @@ PX_set_blob_file(pxdoc_t *pxdoc, const char *filename) {
 		return -1;
 	}
 
+	if(pxdoc->px_stream == NULL) {
+		px_error(pxdoc, PX_RuntimeError, _("Paradox database has not been opened or created when setting the blob file."));
+		return -1;
+	}
+
 	if(pxdoc->px_blob != NULL) {
+		px_error(pxdoc, PX_Warning, _("Blob file has been set already. I will delete the existing one."));
 		PX_delete_blob(pxdoc->px_blob);
 		pxdoc->px_blob = NULL;
 	}
@@ -1839,11 +2016,66 @@ PX_set_blob_file(pxdoc_t *pxdoc, const char *filename) {
 		return -1;
 	}
 
-	if(0 > PX_open_blob_file(pxblob, filename)) {
-		px_error(pxdoc, PX_RuntimeError, _("Could not open blob file."));
+	/* If the paradox database was opend for reading the blob will be too. */
+	if(pxdoc->px_stream->mode == pxfFileRead) {
+		if(0 > PX_open_blob_file(pxblob, filename)) {
+			px_error(pxdoc, PX_RuntimeError, _("Could not open blob file."));
+			return -1;
+		}
+	} else {
+		if(0 > PX_create_blob_file(pxblob, filename)) {
+			px_error(pxdoc, PX_RuntimeError, _("Could not create blob file."));
+			return -1;
+		}
+	}
+
+	pxdoc->px_blob = pxblob;
+
+	return 0;
+}
+/* }}} */
+
+/* PX_set_blob_fp() {{{
+ * Sets the file pointer of the file containing the blobs.
+ */
+PXLIB_API int PXLIB_CALL
+PX_set_blob_fp(pxdoc_t *pxdoc, FILE *fp) {
+	pxblob_t *pxblob;
+
+	if(pxdoc == NULL) {
+		px_error(pxdoc, PX_RuntimeError, _("Did not pass a paradox database."));
 		return -1;
 	}
 
+	if(pxdoc->px_stream == NULL) {
+		px_error(pxdoc, PX_RuntimeError, _("Paradox database has not been opened or created when setting the blob file."));
+		return -1;
+	}
+
+	if(pxdoc->px_blob != NULL) {
+		px_error(pxdoc, PX_Warning, _("Blob file has been set already. I will delete the existing one."));
+		PX_delete_blob(pxdoc->px_blob);
+		pxdoc->px_blob = NULL;
+	}
+
+	if(NULL == (pxblob = PX_new_blob(pxdoc))) {
+		px_error(pxdoc, PX_RuntimeError, _("Could not create new blob file object."));
+		return -1;
+	}
+
+	/* If the paradox database was opend for reading the blob will be too. */
+	if(pxdoc->px_stream->mode == pxfFileRead) {
+		if(0 > PX_open_blob_fp(pxblob, fp)) {
+			px_error(pxdoc, PX_RuntimeError, _("Could not open blob file."));
+			return -1;
+		}
+	} else {
+		if(0 > PX_create_blob_fp(pxblob, fp)) {
+			px_error(pxdoc, PX_RuntimeError, _("Could not create blob file."));
+			return -1;
+		}
+	}
+
 	pxdoc->px_blob = pxblob;
 
 	return 0;
@@ -1873,8 +2105,8 @@ PX_has_blob_file(pxdoc_t *pxdoc) {
 static char*
 _px_read_blobdata(pxblob_t *pxblob, const char *data, int len, int hsize, int *mod, int *blobsize) {
 	int ret;
-	char *blobdata;
-	char head[12];
+	char *blobdata = NULL;
+	unsigned char head[12];
 	pxdoc_t *pxdoc = pxblob->pxdoc;
 	size_t size, offset, mod_nr, index;
 	int leader = len - 10;
@@ -2039,15 +2271,21 @@ PX_get_data_alpha(pxdoc_t *pxdoc, char *data, int len, char **value) {
 		res = recode_buffer_to_buffer(pxdoc->out_recode_request, data, len, &obuf, &olen, &oallocated);
 #else
 #if PX_USE_ICONV
-		size_t ilen = len;
+		size_t ilen;
 		char *iptr, *optr;
-		olen = len + 1;
+		/* Worst case for length of output buffer. If conversion from 1 byte
+		 * to 2 byte chars takes place
+		 */
+		olen = 2*len + 1;
 		/* Do not pxdoc->malloc because the memory is freed with free
 		 * We use free because the memory allocated by recode_buffer_to_buffer()
 		 * is requested with malloc and must be freed with free.
 		 */
 		optr = obuf = (char *) malloc(olen);
 		iptr = data;
+		ilen = 0;
+		while(iptr[ilen] != '\0' && ilen < len)
+			ilen++;
 //		printf("data(%d) = '%s'\n", ilen, data);
 //		printf("obuf(%d) = '%s'\n", olen, obuf);
 		if(0 > (res = iconv(pxdoc->out_iconvcd, &iptr, &ilen, &optr, &olen))) {
@@ -2055,9 +2293,10 @@ PX_get_data_alpha(pxdoc_t *pxdoc, char *data, int len, char **value) {
 			free(obuf);
 			return -1;
 		}
+		*optr = '\0';
 //		printf("data(%d) = '%s'\n", ilen, data);
 //		printf("obuf(%d) = '%s'\n", olen, obuf);
-		olen = len;
+		olen = optr-obuf;
 #endif
 #endif
 	} else {
@@ -2091,16 +2330,19 @@ PX_get_data_alpha(pxdoc_t *pxdoc, char *data, int len, char **value) {
  */
 PXLIB_API int PXLIB_CALL
 PX_get_data_bytes(pxdoc_t *pxdoc, char *data, int len, char **value) {
-	char *obuf = NULL;
-	size_t olen;
-	int res;
-
+	char *buffer;
 	if(data[0] == '\0') {
 //		*value = NULL;
 		return 0;
 	}
 
-	memcpy(*value, data, len);
+	buffer = (char *) pxdoc->malloc(pxdoc, len, _("Allocate memory for field data."));
+	if(!buffer) {
+		*value = NULL;
+		return -1;
+	}
+	memcpy(buffer, data, len);
+	*value = buffer;
 
 	return 1;
 }
@@ -2123,7 +2365,7 @@ PX_get_data_double(pxdoc_t *pxdoc, char *data, int len, double *value) {
 		*value = 0;
 		return 0;
 	}
-	*value = get_double_be(tmp); //*((double *)tmp);
+	*value = get_double_be(tmp); // *((double *)tmp);
 	return 1;
 }
 /* }}} */
@@ -2263,7 +2505,7 @@ PX_get_data_bcd(pxdoc_t *pxdoc, unsigned char *data, int len, char **value) {
 static int
 _px_get_data_blob(pxdoc_t *pxdoc, const char *data, int len, int hsize, int *mod, int *blobsize, char **value) {
 	int ret;
-	char *blobdata;
+	char *blobdata = NULL;
 	char head[20];
 	pxblob_t *pxblob = pxdoc->px_blob;
 	size_t size, offset, mod_nr, index;
@@ -2306,7 +2548,7 @@ _px_get_data_blob(pxdoc_t *pxdoc, const char *data, int len, int hsize, int *mod
 
 	/* Since the blob data is not in the record we will need a blob file */
 	if(!pxblob || !pxblob->mb_stream) {
-		px_error(pxdoc, PX_RuntimeError, _("Blob data is not contained in record and a blob file is not set."));
+		px_error(pxdoc, PX_Warning, _("Blob data is not contained in record and a blob file is not set."));
 		*value = NULL;
 		return -1;
 	}
@@ -2499,10 +2741,6 @@ PX_put_data_alpha(pxdoc_t *pxdoc, char *data, int len, char *value) {
  */
 PXLIB_API void PXLIB_CALL
 PX_put_data_bytes(pxdoc_t *pxdoc, char *data, int len, char *value) {
-	char *obuf = NULL;
-	size_t olen;
-	int res;
-
 	memcpy(data, value, len);
 }
 /* }}} */
@@ -2650,10 +2888,12 @@ PX_put_data_bcd(pxdoc_t *pxdoc, char *data, int len, char *value) {
 /* }}} */
 
 /* _px_put_data_blob() {{{
- * Reads data of blob or graphic into memory and returns a pointer to it.
- * The parameter hsize contains the length of the header right before
- * the blob/graphic in the .MB file. It is 17 Bytes for graphics and 9
- * for all other types of blobs (I'm not completely sure about OLE).
+ * Writes data of blob or graphic from memory into a file.
+ * The parameter data is a pointer to the position in the data record.
+ * len is the space available for the blob in the data record.
+ * value is the pointer to the blob with length valuelen.
+ * The function determines if the blob data fits into the data record.
+ * If it does, it will not need an open blob file.
  */
 static int
 _px_put_data_blob(pxdoc_t *pxdoc, const char *data, int len, char *value, int valuelen) {
@@ -2662,7 +2902,7 @@ _px_put_data_blob(pxdoc_t *pxdoc, const char *data, int len, char *value, int va
 	int leader;
 
 	/* If the (field length - 10) is large enough to hold the blob data,
-	 * we don't need bother writing into the blob file. */
+	 * we don't need to bother writing into the blob file. */
 	leader = len - 10;
 	if(valuelen > leader) {
 		pxblob = pxdoc->px_blob;
@@ -2675,7 +2915,7 @@ _px_put_data_blob(pxdoc_t *pxdoc, const char *data, int len, char *value, int va
 			TMbBlockHeader2 mbbh;
 			int used_blocks;
 
-			fprintf(stderr, "Blob goes into type 2 block\n");
+//			fprintf(stderr, "Blob goes into type 2 block\n");
 			if(pxblob->seek(pxdoc, pxs, (pxblob->used_datablocks+1)*4096, SEEK_SET) < 0) {
 				px_error(pxdoc, PX_RuntimeError, _("Could not go to the begining of the first free block in the blob file."));
 				return -1;
@@ -2794,6 +3034,7 @@ _px_put_data_blob(pxdoc_t *pxdoc, const char *data, int len, char *value, int va
 		else
 			memcpy((char *) data, value, valuelen);
 	}
+	return 0;
 }
 /* }}} */
 
@@ -2801,9 +3042,9 @@ _px_put_data_blob(pxdoc_t *pxdoc, const char *data, int len, char *value, int va
  * Stores bcd number bytes in a data block.
  * len is the number of decimal numbers.
  */
-PXLIB_API void PXLIB_CALL
+PXLIB_API int PXLIB_CALL
 PX_put_data_blob(pxdoc_t *pxdoc, char *data, int len, char *value, int valuelen) {
-	_px_put_data_blob(pxdoc, data, len, value, valuelen);
+	return(_px_put_data_blob(pxdoc, data, len, value, valuelen));
 }
 /* }}} */
 
